/*
 * @Read_MiniSeed.java Version 1.1 03/02/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.applet.*;

import util.utilFileIO;
import seismic.seisStruct;
import seismic.io.seisReadMSeed;

/** Class Read_MiniSeed
 *  <p> This Class is the starting point for read miniseed applet.
 *
 *  @version 1.1 03/02/2015
 *  @author  John R. Victorine
 */

public class Read_MiniSeed extends Applet
{
  private utilFileIO pIO  = null;
  private String     sOut = "";

  private seisStruct st   = null;

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone    = false;
  private String  sKID            = new String("0"); // LAS KID
  private String  sKID_Standalone = new String("0");

  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor Read_MiniSeed()
   *  <p> This is the Constructor for this class.
   */

  public Read_MiniSeed() { }

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Stratigraphic Data Collection and Biostratigraphy Profile
   *      Viewer Module.
   */

  public void init()
  {
    try
    {
      sKID = this.getParameter("sKID", sKID_Standalone);
      if (sKID.equals("undefined"))
        sKID = new String("0");
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    FileDialog dialog    = null;
    Frame      frameTemp = new Frame();

    dialog = new FileDialog( frameTemp, "Select CSV File from your PC." );
    dialog.setVisible( true );
    frameTemp.setVisible( true );

    // Retrieve CSV File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      String sFile = new String(dialog.getDirectory() + "/" + dialog.getFile());

	  // Input miniseed data from file

      seisReadMSeed pIn = new seisReadMSeed();
      pIn.Open(sFile);
      seisStruct st = seismic.seisUtility.copyData( pIn.getData() );
      pIn.Close();

	  // Open Output File

	  openASCIIFile(dialog.getDirectory());

      // Write Out Data

  	  if (st != null)
	  {
        sOut = new String(st.network  + "." +
                          st.station  + "." +
                          st.location + "." +
                          st.channel  + "\n");
        pIO.Write( sOut );

        if (st.iRows > 0)
        {
  	      for (int i=0; i<st.iRows; i++)
	  	  {
            sOut = new String(""+st.ts[i].toString()+", "+st.data[i] + "\n");
	        pIO.Write( sOut );
		  }
	    }
      }

      // Close Output File

      closeASCIIFile();
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /** Method openASCIIFile()
   *  <p> Open ASCII File to write in same directory as input file
   *  @param sPath     = directory path for output the same as input
   */

  public void openASCIIFile(String sPath)
  {
    pIO  = new utilFileIO();

    // Open the File for write

    String sFile = new String("dataOut.CSV");

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFile );
  }

  /** Method closeASCIIFile()
   *  <p> Close ASCII File
   */

  public void closeASCIIFile()
  {
    pIO.Close();
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Info
   */

  public String getAppletInfo() { return ("Digitize E-Logs"); }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo = { {"sKID",  "String", "User Primary Key"}, };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    Read_MiniSeed applet = new Read_MiniSeed();

    applet.isStandalone    = true;
    applet.sKID_Standalone = new String("0");
    if (args.length > 0)
      applet.sKID_Standalone = new String(args[0]);

    applet.init();
    applet.start();
  }
}
