/*
 * @seisUtility.java Version 1.1 03/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import edu.sc.seis.seisFile.mseed.DataRecord;

import seismic.seisStruct;  // Seismic Data Structure

/** Class seisUtility
 *  <p> This is the seismic data structure utility class for
 *      IRIS (Incorporated Research Institutions for Seismology) Data.
 *
 *  @version 1.1 03/03/2015
 *  @author  John R. Victorine
 */

public class seisUtility
{
  /** Method copyList(List<DataRecord> oldList)
   *  <p> This method will copy the IRIS Data Record Array List
   *  @param  oldList = Old IRIS Data Record Array List
   *  @return newList = New IRIS Data Record Array List
   */

  public static List<DataRecord> copyList(List<DataRecord> oldList)
  {
    List<DataRecord> newList = new ArrayList<DataRecord>();

    if (oldList != null)
    {
      for (DataRecord dr : oldList)
      {
        newList.add(dr);
      }
    }

    return (newList);
  }

  /** Method seisStruct copyData(seisStruct oldStruct)
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct copyData(seisStruct stOld)
  {
	seisStruct stNew = null;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.tsStart  = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum = stOld.dMaximum; // Maximum Data Value
      stNew.dMinimum = stOld.dMinimum; // Minimum Data Value

      // Seismic Records

      stNew.iRows = stOld.iRows;                   // Total Number of rows
      stNew.ts    = new Timestamp[stNew.iRows];    // Time Stamp of data record
      stNew.data  = new double[stNew.iRows];       // Data record

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]   = new Timestamp(stOld.ts[i].getTime());
		  stNew.data[i] = stOld.data[i];
		}
	  }
	}

	return (stNew);
  }

}

/*
 *  @version 1.1 03/03/2015
 *  @author  John Victorine
 */
